﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using DistributedGA.Core.Domain;
using DistributedGA.Core.Interface;
using System.Timers;

namespace DistributedGA.Core.Implementation {
  public class WcfPeerListManager : IPeerListManager {

    private string serverString = null;

    private PeerInfo myself = null;

    private Timer timer = null; //sends heartbeat to contact-server

    private Object timerLock = new Object();

    private ChannelFactory<IContactService> myChannelFactory;

    private IContactService client;

    private IContactService heartbeatClient;



    public void Init(PeerInfo source, string contactServerUrl) {
      serverString = contactServerUrl;
      myself = source;

      //Init ChannelFactory and Clients
      var binding = new NetTcpBinding();
      var endpoint = new EndpointAddress(serverString);
      myChannelFactory = new ChannelFactory<IContactService>(binding, endpoint);
      client = myChannelFactory.CreateChannel();
      heartbeatClient = myChannelFactory.CreateChannel();
      //Register Peer
      client.RegisterPeer(source);
      //Start heartbeat timer
      timer = new Timer(1000 * 20); //each 20 seconds
      timer.Elapsed += SendHeartbeatToServer;
      timer.Start();
    }

    public List<PeerInfo> GetPeerList() {
      try {
        var allPeers = client.GetPeerList(myself); //maybe timout exception...
        return allPeers;
      }
      catch { } //if maybe sending failed (because of connection lost, etc.): just ignore
      return new List<PeerInfo>();
    }

    public void SendLogToServer(string msg) {
      client.MakeLog(myself, msg);
    }

    public void Dispose() {
      timer.Stop();
      timer.Dispose();
      ((IClientChannel)client).Close();
      ((IClientChannel)heartbeatClient).Close();
      myChannelFactory.Close();
      client = null;
      myChannelFactory = null;
    }

    private void SendHeartbeatToServer(object sender, ElapsedEventArgs e) {
      lock (timerLock) {
        try {
          heartbeatClient.UpdateHeartbeat(myself);
        }
        catch { } //nothing to do
      }
    }

  }
}
